@@ -1,5 +1,6 @@
 #include <config.h>
 #include <string.h>
+#include <unistd.h>
 #include <glib/gthread.h>
 #include <glib/gqueue.h>
 #include <gtk/gtkmain.h>
@@ -20,6 +21,8 @@
 #include "eog-image-private.h"
 #include "eog-pixbuf-util.h"
 #include "eog-image-cache.h"
+#include "eog-metadata-reader.h"
+#include "eog-image-save-info.h"
 #if HAVE_JPEG
 #include "eog-image-jpeg.h"
 #endif
@@ -36,7 +39,6 @@ enum {
 	SIGNAL_LOADING_UPDATE,
 	SIGNAL_LOADING_SIZE_PREPARED,
 	SIGNAL_LOADING_FINISHED,
-	SIGNAL_LOADING_INFO_FINISHED,
 	SIGNAL_LOADING_FAILED,
 	SIGNAL_LOADING_CANCELLED,
 	SIGNAL_PROGRESS,
@@ -394,14 +396,6 @@ eog_image_class_init (EogImageClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);			     
-	eog_image_signals [SIGNAL_LOADING_INFO_FINISHED] = 
-		g_signal_new ("loading_info_finished",
-			      G_TYPE_OBJECT,
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EogImageClass, loading_info_finished),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE, 0);			     
 	eog_image_signals [SIGNAL_LOADING_FAILED] = 
 		g_signal_new ("loading_failed",
 			      G_TYPE_OBJECT,
@@ -475,6 +469,7 @@ eog_image_instance_init (EogImage *img)
 	priv->width = priv->height = -1;
 	priv->modified = FALSE;
 	priv->status_mutex = g_mutex_new ();
+	priv->load_finished = NULL;
 #if HAVE_EXIF
 	priv->exif = NULL;
 #endif
@@ -531,83 +526,48 @@ eog_image_error_quark (void)
 	return q;
 }
 
+#if HAVE_EXIF
+static void
+update_exif_data (EogImage *image)
+{
+	EogImagePrivate *priv;
+	ExifEntry *entry;
+	ExifByteOrder bo;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+
+	priv = image->priv;
+
+#ifdef DEBUG
+	g_message ("update exif data");
+#endif
+	if (priv->exif == NULL) return;
+
+	/* FIXME: Must we update more properties here? */
+	bo = exif_data_get_byte_order (priv->exif);
+	
+	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_PIXEL_X_DIMENSION);
+	if (entry != NULL) {
+		exif_set_long (entry->data, bo, priv->width);
+	}
+	
+	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_PIXEL_Y_DIMENSION);
+	if (entry != NULL) {
+		exif_set_long (entry->data, bo, priv->height);
+	}
+}
+
+	
+#endif 
+
 static gboolean
-check_load_status (gpointer data)
+load_emit_signal_progress (gpointer data)
 {
 	EogImage *img;
-	EogImagePrivate *priv;
-	int load_status;
-	int x, y, width, height;
-	gboolean call_again = TRUE;
-	float progress;
 
 	img = EOG_IMAGE (data);
-	priv = img->priv;
-	
-	g_mutex_lock (priv->status_mutex);
 
-	g_source_remove (priv->load_id);
-	priv->load_id = -1;
-
-	load_status = priv->load_status;
-	x = priv->update_x1;
-	y = priv->update_y1;
-	width = priv->update_x2 - x;
-	height = priv->update_y2 - y;
-	progress = priv->progress;
-
-	priv->load_status = 0;
-	priv->update_x1 = 10000;
-	priv->update_y1 = 10000;
-	priv->update_x2 = 0;
-	priv->update_y2 = 0;
-
-	g_mutex_unlock (priv->status_mutex);
-
-	if ((load_status & EOG_IMAGE_LOAD_STATUS_FAILED) > 0) {
-		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FAILED], 0, priv->error_message);
-		call_again = FALSE;
-	}
-	
-	if ((load_status & EOG_IMAGE_LOAD_STATUS_CANCELLED) > 0) {
-		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_CANCELLED], 0);
-		call_again = FALSE;
-	}
-	
-	if ((load_status & EOG_IMAGE_LOAD_STATUS_PREPARED) > 0) {
-		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_SIZE_PREPARED], 0, priv->width, priv->height);
-	}
-	
-	if ((load_status & EOG_IMAGE_LOAD_STATUS_UPDATED) > 0) {
-		g_signal_emit (img, eog_image_signals [SIGNAL_LOADING_UPDATE], 0, 
-			       x, y, width, height);
-	}
-
-	if ((load_status & EOG_IMAGE_LOAD_STATUS_INFO_DONE) > 0) {
-		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_INFO_FINISHED], 0);
-	}
-	
- 	if ((load_status & EOG_IMAGE_LOAD_STATUS_DONE) > 0) {
-		if ((load_status & EOG_IMAGE_LOAD_STATUS_TRANSFORMED) > 0) {
-			g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_IMAGE_CHANGED], 0);
-		}
-		else {
-			g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FINISHED], 0);
-		}
-
-		call_again = FALSE;
-	}
-
- 	if ((load_status & EOG_IMAGE_LOAD_STATUS_PROGRESS) > 0) {
-		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_PROGRESS], 0, progress);
-	}
-
-	if (call_again) {
-		priv->load_id = g_timeout_add (CHECK_LOAD_TIMEOUT, check_load_status, img);
-	}
-	else {
-		g_object_unref (img);
-	}
+	g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_PROGRESS], 0, img->priv->progress);
 
 	return FALSE;
 }
@@ -622,104 +582,113 @@ transform_progress_hook_async (EogTransform *trans, float progress, gpointer hoo
 	img = EOG_IMAGE(hook_data);
 	priv = img->priv;
 
-	if (progress != priv->progress) {
+	if ((progress < priv->progress) || (progress - priv->progress) > 0.1) {
 		g_mutex_lock (priv->status_mutex);
 		priv->progress = progress;
-		priv->load_status |= EOG_IMAGE_LOAD_STATUS_PROGRESS;
 		g_mutex_unlock (priv->status_mutex);		
+
+		g_idle_add (load_emit_signal_progress, img);
 	}
 }
 
-static void 
-load_area_updated (GdkPixbufLoader *loader, gint x, gint y, gint width, gint height, gpointer data)
+
+static gboolean
+load_emit_signal_done (gpointer data)
 {
 	EogImage *img;
 	EogImagePrivate *priv;
-	int x2, y2;
 
 	img = EOG_IMAGE (data);
-
-#ifdef DEBUG
-	g_print ("load_area_updated\n");
-#endif
-
 	priv = img->priv;
 
-	g_mutex_lock (priv->status_mutex);
-	priv->load_status |= EOG_IMAGE_LOAD_STATUS_UPDATED;
+	switch (img->priv->status) {
+	case EOG_IMAGE_STATUS_FAILED:
+	case EOG_IMAGE_STATUS_UNKNOWN:
+		if (priv->image != NULL) {
+			g_object_unref (priv->image);
+			priv->image = NULL;
+		}
+#if HAVE_EXIF
+		if (priv->exif != NULL) {
+			exif_data_unref (priv->exif);
+			priv->exif = NULL;
+		}
+#endif
+		priv->progress = 0.0;
 
-	if (priv->image == NULL) {
-		priv->image = gdk_pixbuf_loader_get_pixbuf (loader);
-		g_object_ref (priv->image);
+		if (img->priv->status == EOG_IMAGE_STATUS_FAILED) {
+			g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FAILED], 0, priv->error_message);
+		}
+		else {
+			g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_CANCELLED], 0);
 	}
 	
-	x2 = x + width;
-	y2 = y + height;
+		break;
 
-	priv->update_x1 = MIN (priv->update_x1, x);
-	priv->update_y1 = MIN (priv->update_y1, y);
-	priv->update_x2 = MAX (priv->update_x2, x2);
-	priv->update_y2 = MAX (priv->update_y2, y2);
+	case EOG_IMAGE_STATUS_LOADED:
+		g_assert (priv->file_type != NULL);
 
-	g_mutex_unlock (priv->status_mutex);
+		priv->progress = 1.0;
+		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FINISHED], 0);
+		break;
 
-#ifdef DEBUG
-	g_print ("area_updated: x: %i, y: %i, width: %i, height: %i\n", x, y, width, height);
-#endif
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+	g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_PROGRESS], 0, img->priv->progress);
+
+
+	return FALSE;
 }
 
+static gboolean
+load_emit_signal_size_prepared (gpointer data)
+{
+
+	EogImage *img;
+	
+	img = EOG_IMAGE (data);
+
+	g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_SIZE_PREPARED], 0, img->priv->width, img->priv->height);
+
+	return FALSE;
+	}
+
 static void
 load_size_prepared (GdkPixbufLoader *loader, gint width, gint height, gpointer data)
 {
 	EogImage *img;
 
 	g_return_if_fail (EOG_IS_IMAGE (data));
-	
+
 	img = EOG_IMAGE (data);
 
 	g_mutex_lock (img->priv->status_mutex);
-	img->priv->load_status |= EOG_IMAGE_LOAD_STATUS_PREPARED;
-
 	img->priv->width = width;
 	img->priv->height = height;
 	g_mutex_unlock (img->priv->status_mutex);
+
+	g_idle_add (load_emit_signal_size_prepared, img);
 }
 
-#if HAVE_EXIF
-
-static void
-update_exif_data (EogImage *image)
+static EogMetadataReader*
+check_for_metadata_img_format (EogImage *img, guchar *buffer, int bytes_read)
 {
-	EogImagePrivate *priv;
-	ExifEntry *entry;
-	ExifByteOrder bo;
-
-	g_return_if_fail (EOG_IS_IMAGE (image));
+	EogMetadataReader *md_reader = NULL;
 	
-	priv = image->priv;
+	g_print ("check img format for jpeg: %x%x - length: %i\n", buffer[0], buffer[1], bytes_read);
 	
-#ifdef DEBUG
-	g_message ("update exif data");
-#endif
-	if (priv->exif == NULL) return;
-
-	/* FIXME: Must we update more properties here? */
-	bo = exif_data_get_byte_order (priv->exif);
-
-	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_PIXEL_X_DIMENSION);
-	if (entry != NULL) {
-		exif_set_long (entry->data, bo, priv->width);
+	if (bytes_read >= 2) {
+		/* SOI (start of image) marker for JPEGs is 0xFFD8 */
+		if ((buffer[0] == 0xFF) && (buffer[1] == 0xD8)) {		
+			md_reader = eog_metadata_reader_new (EOG_METADATA_JPEG);
+		}
 	}
-
-	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_PIXEL_Y_DIMENSION);
-	if (entry != NULL) {
-		exif_set_long (entry->data, bo, priv->height);
-	}	
+	
+	return md_reader;
 }
 
-
-#endif 
-
 /* this function runs in it's own thread */
 static gpointer
 real_image_load (gpointer data)
@@ -735,9 +704,8 @@ real_image_load (gpointer data)
 	GnomeVFSFileSize bytes_read_total;
 	gboolean failed;
 	GError *error = NULL;
-#if HAVE_EXIF
-	ExifLoader *exif_loader;
-#endif
+	gboolean first_run = TRUE;
+	EogMetadataReader *md_reader = NULL;
 
 	img = EOG_IMAGE (data);
 	priv = img->priv;
@@ -747,12 +715,7 @@ real_image_load (gpointer data)
 #endif
 
 	g_assert (priv->image == NULL);
-#if HAVE_EXIF
-	if (priv->exif != NULL) {
-		/* FIXME: this shouldn't happen but do so sometimes. */
-		g_warning ("exif data not freed\n");
-	}
-#endif
+
 	if (priv->file_type != NULL) {
 		g_free (priv->file_type);
 		priv->file_type = NULL;
@@ -761,10 +724,12 @@ real_image_load (gpointer data)
 	result = gnome_vfs_open_uri (&handle, priv->uri, GNOME_VFS_OPEN_READ);
 	if (result != GNOME_VFS_OK) {
 		g_mutex_lock (priv->status_mutex);
-		priv->load_status |= EOG_IMAGE_LOAD_STATUS_FAILED;
+		priv->status = EOG_IMAGE_STATUS_FAILED;
 		priv->error_message = (char*) gnome_vfs_result_to_string (result);
 		g_mutex_unlock (priv->status_mutex);
 
+		g_idle_add (load_emit_signal_done, img);
+
 		return NULL;
 	}
 
@@ -776,32 +741,27 @@ real_image_load (gpointer data)
 					      GNOME_VFS_FILE_INFO_DEFAULT);
 	if ((result != GNOME_VFS_OK) || (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) == 0) {
 		g_mutex_lock (priv->status_mutex);
-		priv->load_status |= EOG_IMAGE_LOAD_STATUS_FAILED;
 		priv->error_message = (char*) gnome_vfs_result_to_string (result);
+		priv->status = EOG_IMAGE_STATUS_FAILED;
 		g_mutex_unlock (priv->status_mutex);
 
 		gnome_vfs_file_info_unref (info);
 
+		g_idle_add (load_emit_signal_done, img);
+
 		return NULL;
 	}
 	bytes_read_total = 0;
-	priv->progress = 0.0;
 	priv->bytes = info->size;
 	
 	buffer = g_new0 (guchar, READ_BUFFER_SIZE);
 	loader = gdk_pixbuf_loader_new ();
 	failed = FALSE;
-#if HAVE_EXIF
-	exif_loader = exif_loader_new ();
-#endif
 
 	g_signal_connect_object (G_OBJECT (loader), "size-prepared", (GCallback) load_size_prepared, img, 0);
-	if (priv->mode == EOG_IMAGE_LOAD_PROGRESSIVE) {
-		g_signal_connect_object (G_OBJECT (loader), "area-updated", (GCallback) load_area_updated, img, 0);
-	}
 
 	while (!priv->cancel_loading) {
-		float new_progress;
+	        GnomeVFSFileSize last_progress_bytes = 0;
 		
 		result = gnome_vfs_read (handle, buffer, READ_BUFFER_SIZE, &bytes_read);
 		if (result == GNOME_VFS_ERROR_EOF || bytes_read == 0) {
@@ -818,43 +778,37 @@ real_image_load (gpointer data)
 		}
 
 		bytes_read_total += bytes_read;
-		new_progress = (float) bytes_read_total / (float) info->size;
-		if (priv->progress != new_progress) {
-			priv->progress = new_progress;
-			priv->load_status |= EOG_IMAGE_LOAD_STATUS_PROGRESS;
+		if ((bytes_read_total - last_progress_bytes) > 250000) { /* update progress each ~250kb */
+			priv->progress = (float) bytes_read_total / (float) info->size;
+			last_progress_bytes = bytes_read_total;
+			g_idle_add (load_emit_signal_progress, img);
 		}
-
-#if HAVE_EXIF
-		if (exif_loader != NULL && (exif_loader_write (exif_loader, buffer, bytes_read) == 0)) {
-			g_mutex_lock (img->priv->status_mutex);
-			priv->exif = exif_loader_get_data (exif_loader);
-			priv->load_status |= EOG_IMAGE_LOAD_STATUS_INFO_DONE;
-			g_mutex_unlock (img->priv->status_mutex);
-			
-			exif_loader_unref (exif_loader);
-			exif_loader = NULL;
+		
+		/* check if we support reading metadata for that image format (only JPG atm) */
+		if (first_run) {
+			md_reader = check_for_metadata_img_format (img, buffer, bytes_read);
+			first_run = FALSE;
+		}
+		
+		if (md_reader != NULL) {
+			eog_metadata_reader_consume (md_reader, buffer, bytes_read);
 		}
-#endif
 	}
 
 	gdk_pixbuf_loader_close (loader, NULL);	
-
+	
 	g_free (buffer);
 	gnome_vfs_close (handle);
 	gnome_vfs_file_info_unref (info);
 	
-	if (failed) {
+	if (failed || (bytes_read_total == 0)) {
 		g_mutex_lock (priv->status_mutex);
-		if (priv->image != NULL) {
-			g_object_unref (priv->image);
-			priv->image = NULL;
-		}
-		priv->load_status |= EOG_IMAGE_LOAD_STATUS_FAILED;
-		priv->progress = 0.0;
-		
 		if (error != NULL) {
 			priv->error_message = g_strdup (error->message);
 		}
+		else if (bytes_read_total == 0) {
+			priv->error_message = g_strdup (_("empty file"));
+		}
 		else {
 			priv->error_message = NULL;
 		}
@@ -863,20 +817,7 @@ real_image_load (gpointer data)
 	}
 	else if (priv->cancel_loading) {
 		g_mutex_lock (priv->status_mutex);
-		if (priv->image != NULL) {
-			g_object_unref (priv->image);
-			priv->image = NULL;
-		}
-#if HAVE_EXIF
-		if (priv->exif != NULL) {
-			exif_data_unref (priv->exif);
-			priv->exif = NULL;
-		}
-#endif
 		priv->cancel_loading = FALSE;
-		priv->progress = 0.0;
-
-		priv->load_status |= EOG_IMAGE_LOAD_STATUS_CANCELLED;
 		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 		g_mutex_unlock (priv->status_mutex);
 	}
@@ -913,27 +854,41 @@ real_image_load (gpointer data)
 		if (format != NULL) {
 			priv->file_type = g_strdup (gdk_pixbuf_format_get_name (format));
 		}
-#if HAVE_EXIF
-		update_exif_data (img);
-#endif 
+			
+		if (md_reader != NULL) {
+#if HAVE_EXIF	
+			priv->exif = eog_metadata_reader_get_exif_data (md_reader);
+			priv->exif_chunk = NULL;
+			priv->exif_chunk_len = 0;
+			update_exif_data (img);
+#else
+			eog_metadata_reader_get_exif_chunk (md_reader, &priv->exif_chunk, &priv->exif_chunk_len);
+#endif
+		}
 		eog_image_cache_add (img);
 
-		if (trans != NULL && priv->mode == EOG_IMAGE_LOAD_PROGRESSIVE) {
-			priv->load_status |= EOG_IMAGE_LOAD_STATUS_TRANSFORMED;
-		}
-
-		priv->load_status |= EOG_IMAGE_LOAD_STATUS_DONE;
 		priv->status = EOG_IMAGE_STATUS_LOADED;
 		g_mutex_unlock (priv->status_mutex);
+		
 	}
 
+	g_idle_add (load_emit_signal_done, img);
+
 	if (error != NULL) {
 		g_error_free (error);
 	}
 	g_object_unref (loader);
+	if (md_reader != NULL) {
+		g_object_unref (md_reader);
+		md_reader = NULL;
+	}	
 
 	g_mutex_lock (priv->status_mutex);
 	priv->load_thread = NULL;
+
+	if (priv->load_finished != NULL) {
+		g_cond_broadcast (priv->load_finished);
+	}
 	g_mutex_unlock (priv->status_mutex);
 
 	return NULL;
@@ -952,7 +907,6 @@ eog_image_load (EogImage *img, EogImageLoadMode mode)
 		g_assert (priv->image != NULL);
 		eog_image_cache_reload (img);
 		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FINISHED], 0);
-		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_INFO_FINISHED], 0);
 	}
 	else if (priv->status == EOG_IMAGE_STATUS_FAILED) {
 		g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_LOADING_FAILED], 0, "");		
@@ -960,17 +914,15 @@ eog_image_load (EogImage *img, EogImageLoadMode mode)
 	else if (priv->status == EOG_IMAGE_STATUS_UNKNOWN) { 
 		g_assert (priv->image == NULL);
 		
-                /* make sure the object isn't destroyed while we try to load it */
+		/* make sure the object isn't destroyed while we try to load it */
 		g_object_ref (img);
 
 		/* initialize data fields for progressive loading */
-		priv->load_status = EOG_IMAGE_LOAD_STATUS_NONE;
-		priv->update_x1 = 10000;
-		priv->update_y1 = 10000;
-		priv->update_x2 = 0;
-		priv->update_y2 = 0;
 		priv->error_message = NULL;
 		priv->cancel_loading = FALSE;
+		priv->mode = EOG_IMAGE_LOAD_COMPLETE;
+		
+#if 0
 		priv->mode = mode;
 
 		if (priv->mode == EOG_IMAGE_LOAD_DEFAULT) {
@@ -1005,14 +957,57 @@ eog_image_load (EogImage *img, EogImageLoadMode mode)
 				
 			}
 		}
+#endif
 
 		/* start the thread machinery */
 		priv->status      = EOG_IMAGE_STATUS_LOADING;
-		priv->load_id     = g_timeout_add (CHECK_LOAD_TIMEOUT, check_load_status, img);
 		priv->load_thread = g_thread_create (real_image_load, img, TRUE, NULL);
 	}
 }
 
+gboolean
+eog_image_load_sync (EogImage *img, EogImageLoadMode mode)
+{
+	EogImagePrivate *priv;
+	gboolean success = FALSE; 
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+
+	priv = img->priv;
+
+	if (priv->status == EOG_IMAGE_STATUS_LOADED) {
+		g_assert (priv->image != NULL);
+		eog_image_cache_reload (img);
+		success = TRUE;
+	}
+	else if (priv->status == EOG_IMAGE_STATUS_FAILED) {
+		success = FALSE;
+	}
+	else if (priv->status == EOG_IMAGE_STATUS_UNKNOWN) { 
+
+		g_mutex_lock (priv->status_mutex);
+
+		priv->load_finished = g_cond_new ();
+		
+		eog_image_load (img, mode);
+		g_cond_wait (priv->load_finished, priv->status_mutex);
+		
+		g_cond_free (priv->load_finished);
+		priv->load_finished = NULL;
+
+		success = (priv->status == EOG_IMAGE_STATUS_LOADED);
+	
+		g_mutex_unlock (priv->status_mutex);
+	}
+
+
+	if (success) {
+		g_assert (priv->file_type != NULL);
+	}
+
+	return success;
+}
+
 gboolean 
 eog_image_load_thumbnail (EogImage *img)
 {
@@ -1210,82 +1205,313 @@ eog_image_undo (EogImage *img)
 	priv->modified = (priv->undo_stack != NULL);
 }
 
-/* is_local_uri: 
- * 
- * Checks if the URI points to a local file system. This tests simply
- * if the URI scheme is 'file'. This function is used to ensure that
- * we can write to the path-part of the URI with non-VFS aware
- * filesystem calls.
- */
-static gboolean 
-is_local_uri (const GnomeVFSURI* uri)
+static char*
+tmp_file_get_path (void)
 {
-	const char *scheme;
+	char *tmp_file;
+	int fd;
 
-	g_return_val_if_fail (uri != NULL, FALSE);
+	tmp_file = g_build_filename (g_get_tmp_dir (), "eog-save-XXXXXX", NULL);
+	fd = g_mkstemp (tmp_file);
+	if (fd == -1) {
+		/* error case */
+		g_free (tmp_file);
+		tmp_file = NULL;
+	}
+	else {
+		close (fd);
+	}
 
-	scheme = gnome_vfs_uri_get_scheme (uri);
-	return (g_strcasecmp (scheme, "file") == 0);
+	return tmp_file;
 }
 
-static char*
-get_save_file_type_by_suffix (const char *local_path)
+static gboolean
+tmp_file_move_to_uri (const char* tmpfile, const GnomeVFSURI *uri, gboolean overwrite, GError **error)
 {
-	GSList *savable_formats = NULL;
-	GSList *it;
-	char *file_type = NULL;
-	char *suffix;
-	GdkPixbufFormat *format;
-	char **extension;
-	int i;
+	GnomeVFSResult result;
+	GnomeVFSURI *source_uri;
+	GnomeVFSFileInfo *info;
+	GnomeVFSXferOverwriteMode overwrt_mode = GNOME_VFS_XFER_OVERWRITE_MODE_ABORT;
 
-	/* FIXME: this is probably not unicode friendly */
-	suffix = g_strrstr (local_path, ".");
-	if (suffix == NULL) {
-		return NULL;
+	if (!overwrite && gnome_vfs_uri_exists ((GnomeVFSURI*) uri)) 
+	{
+		/* explicit check if uri exists, seems that gnome_vfs_xfer_uri, doesn't
+		 *  work as expected 
+		 */
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_FILE_EXISTS,
+			     _("File exists"));
+		return FALSE;
 	}
 
-	/* skip '.' from the suffix string */
-	if (strlen (suffix) > 1) {
-		suffix++;
+	info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info_uri ((GnomeVFSURI*) uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+	if (result != GNOME_VFS_OK) {
+		/* we don't propagate the error here, because if we get a 
+		 * fatal error, the xfer_uri will fail too and then
+		 * handled.
+		 */
+		gnome_vfs_file_info_unref (info);
+		info = NULL;
 	}
 
-	savable_formats = eog_pixbuf_get_savable_formats ();
+	if (overwrite == TRUE) {
+		overwrt_mode = GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE;
+	}
+	source_uri = gnome_vfs_uri_new (tmpfile);
 
-	/* iterate over the availabe formats and check for every
-	 * possible format suffix if it matches the file suffix */
-	for (it = savable_formats; it != NULL && file_type == NULL; it = it->next) {
+	result = gnome_vfs_xfer_uri (source_uri,
+				     uri, 
+				     GNOME_VFS_XFER_DELETE_ITEMS,           /* delete source file */
+				     GNOME_VFS_XFER_ERROR_MODE_ABORT,       /* abort on all errors */
+				     overwrt_mode,
+				     NULL,                                  /* no progress callback */
+				     NULL);
 
-		format = (GdkPixbufFormat*) it->data;
-		extension = gdk_pixbuf_format_get_extensions (format);
-		
-		for (i = 0; extension[i] != NULL && file_type == NULL; i++) {
-			if (g_ascii_strcasecmp (extension[i], suffix) == 0) {
-				file_type = gdk_pixbuf_format_get_name (format);
-			}
+	gnome_vfs_uri_unref (source_uri);
+
+	if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_FILE_EXISTS,
+			     gnome_vfs_result_to_string (result));
+	}
+	else if (result != GNOME_VFS_OK) {
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_VFS, 
+			     gnome_vfs_result_to_string (result));
+	}
+	else if (info != NULL) {
+		/* reset file permissions/owner to the original ones */
+		GnomeVFSSetFileInfoMask mask = 
+			GNOME_VFS_SET_FILE_INFO_PERMISSIONS | GNOME_VFS_SET_FILE_INFO_OWNER;
+		gnome_vfs_set_file_info_uri ((GnomeVFSURI*) uri, info, mask);
+	}
+
+	if (info != NULL) {
+		gnome_vfs_file_info_unref (info);
+	}
+
+	return (result == GNOME_VFS_OK);
+}
+
+static gboolean
+tmp_file_delete (char *tmpfile)
+{
+	if (tmpfile == NULL) return FALSE;
+
+	if (g_file_test (tmpfile, G_FILE_TEST_EXISTS)) {
+		int result;
+
+		result = unlink (tmpfile);
+		if (result == -1) {
+			g_warning ("Couldn't delete temporary file: %s\n", tmpfile);
+			return FALSE;
 		}
-
-		g_strfreev (extension);
 	}
-	g_slist_free (savable_formats);
 
-	/* file_type is either NULL or contains the name of the format */
-	return file_type;
+	return TRUE;
+}
+
+static void 
+eog_image_reset_modifications (EogImage *image)
+{
+	EogImagePrivate *priv;
+	GList *it = NULL;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+
+	priv = image->priv;
+
+	/* free the undo stack */
+	for (it = priv->undo_stack; it != NULL; it = it->next) {
+		g_object_unref (G_OBJECT (it->data));
+	}
+	g_list_free (priv->undo_stack);
+	priv->undo_stack = NULL;
+
+	/* free accumulated transform object */
+	if (priv->trans != NULL) {
+		g_object_unref (priv->trans);
+		priv->trans = NULL;
+	}
+	priv->modified = FALSE;
+}
+
+static void
+eog_image_link_with_target (EogImage *image, EogImageSaveInfo *target)
+{
+	EogImagePrivate *priv;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (EOG_IS_IMAGE_SAVE_INFO (target));
+
+	priv = image->priv;
+
+	/* update file location */
+	if (priv->uri != NULL) {
+		gnome_vfs_uri_unref (priv->uri);
+	}
+	priv->uri = gnome_vfs_uri_ref (target->uri);
+
+	/* Clear caption and caption key, these will be 
+	 * updated on next eog_image_get_caption call.
+	 */
+	if (priv->caption != NULL) {
+		g_free (priv->caption);
+		priv->caption = NULL;
+	}
+	if (priv->caption_key != NULL) {
+		g_free (priv->caption_key);
+		priv->caption_key = NULL;
+	}
+
+	/* update file format */
+	if (priv->file_type != NULL) {
+		g_free (priv->file_type);
+	}
+	priv->file_type = g_strdup (target->format);
 }
 
 gboolean
-eog_image_save (EogImage *img, const GnomeVFSURI *uri, GError **error)
+eog_image_save_by_info (EogImage *img, EogImageSaveInfo *source, GError **error) 
 {
 	EogImagePrivate *priv;
-	char *file;
-	char *target_type = NULL;
-	char *source_type = NULL;
-	gboolean result = FALSE;
-	gboolean source_is_local;
+	gboolean success = FALSE;
+	char *tmpfile;
 
 	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
-	g_return_val_if_fail (uri != NULL, FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+	g_return_val_if_fail (EOG_IS_IMAGE_SAVE_INFO (source), FALSE);
+
+	priv = img->priv;
+
+	/* see if we need any saving at all */
+	if (source->exists && !source->modified) {
+		return TRUE;
+	}
+
+	/* fail if there is no image to save */
+	if (priv->image == NULL) {
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_NOT_LOADED,
+			     _("No image loaded."));
+		return FALSE;
+	}
+
+	/* generate temporary file name */
+	tmpfile = tmp_file_get_path ();
+	if (tmpfile == NULL) {
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_TMP_FILE_FAILED,
+			     _("Temporary file creation failed."));
+		return FALSE;
+	}
+	
+	/* determine kind of saving */
+	if ((g_ascii_strcasecmp (source->format, EOG_FILE_FORMAT_JPEG) == 0) && 
+	    source->exists && source->modified) 
+	{
+		success = eog_image_jpeg_save_file (img, tmpfile, source, NULL, error);
+	}
+
+	if (!success && (*error == NULL)) {
+		success = gdk_pixbuf_save (priv->image, tmpfile, source->format, error, NULL);
+	}
+
+	if (success) {
+		/* try to move result file to target uri */
+		success = tmp_file_move_to_uri (tmpfile, priv->uri, TRUE /*overwrite*/, error);
+	}
+
+	if (success) {
+		eog_image_reset_modifications (img);
+	}
+
+	tmp_file_delete (tmpfile);
+
+	g_free (tmpfile);
+
+	return success;
+}
+
+static gboolean
+eog_image_copy_file (EogImageSaveInfo *source, EogImageSaveInfo *target, GError **error)
+{
+	GnomeVFSResult    result;
+	GnomeVFSFileInfo *info;
+	GnomeVFSXferOverwriteMode overwrt_mode = GNOME_VFS_XFER_OVERWRITE_MODE_ABORT;
+
+	g_return_val_if_fail (EOG_IS_IMAGE_SAVE_INFO (source), FALSE);
+	g_return_val_if_fail (EOG_IS_IMAGE_SAVE_INFO (target), FALSE);
+
+	if (target->overwrite != TRUE && 
+	    gnome_vfs_uri_exists (target->uri)) 
+	{
+		/* explicit check if uri exists, seems that gnome_vfs_xfer_uri, doesn't
+		 *  work as expected 
+		 */
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_FILE_EXISTS,
+			     _("File exists"));
+		return FALSE;
+	}
+	else if (target->overwrite == TRUE) {
+		overwrt_mode = GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE;
+	}
+
+	info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info_uri ((GnomeVFSURI*) target->uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+	if (result != GNOME_VFS_OK) {
+		/* we don't propagate the error here, because if we get a 
+		 * fatal error, the xfer_uri will fail too and then
+		 * handled.
+		 */
+		gnome_vfs_file_info_unref (info);
+		info = NULL;
+	}
+
+	result = gnome_vfs_xfer_uri (source->uri,
+				     target->uri, 
+				     GNOME_VFS_XFER_DEFAULT,            /* copy the data */
+				     GNOME_VFS_XFER_ERROR_MODE_ABORT,   /* abort on all errors */
+				     overwrt_mode,
+				     NULL,                              /* no progress callback */
+				     NULL);
+
+	if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_FILE_EXISTS,
+			     gnome_vfs_result_to_string (result));
+	}
+	else if (result != GNOME_VFS_OK) {
+		g_set_error (error, EOG_IMAGE_ERROR,
+			     EOG_IMAGE_ERROR_VFS, 
+			     gnome_vfs_result_to_string (result));
+	}
+	else if (info != NULL) {
+		/* reset file permissions/owner to the original ones */
+		GnomeVFSSetFileInfoMask mask = 
+			GNOME_VFS_SET_FILE_INFO_PERMISSIONS | GNOME_VFS_SET_FILE_INFO_OWNER;
+		gnome_vfs_set_file_info_uri (target->uri, info, mask);
+	}
+
+	if (info != NULL) {
+		gnome_vfs_file_info_unref (info);
+	}
+
+	return (result == GNOME_VFS_OK);
+}
+
+gboolean
+eog_image_save_as_by_info (EogImage *img, EogImageSaveInfo *source, EogImageSaveInfo *target, GError **error)
+{
+	EogImagePrivate *priv;
+	gboolean success = FALSE;
+	char *tmpfile;
+	gboolean direct_copy = FALSE;
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+	g_return_val_if_fail (EOG_IS_IMAGE_SAVE_INFO (source), FALSE);
+	g_return_val_if_fail (EOG_IS_IMAGE_SAVE_INFO (target), FALSE);
 
 	priv = img->priv;
 
@@ -1297,106 +1523,76 @@ eog_image_save (EogImage *img, const GnomeVFSURI *uri, GError **error)
 		return FALSE;
 	}
 
-	/* Since we rely on some local system calls resp. non-VFS aware libs, 
-	 * we can write only to local files.
-	 */
-	if (!is_local_uri (uri)) {
+	/* generate temporary file name */
+	tmpfile = tmp_file_get_path ();
+	if (tmpfile == NULL) {
 		g_set_error (error, EOG_IMAGE_ERROR,
-			     EOG_IMAGE_ERROR_NOT_LOADED,
-			     _("Images can only be saved as local files."));
+			     EOG_IMAGE_ERROR_TMP_FILE_FAILED,
+			     _("Temporary file creation failed."));
 		return FALSE;
 	}
 	
-	/* find file type for saving, according to filename suffix */
-	file = (char*) gnome_vfs_uri_get_path (uri); /* don't free file, it's a static string */
-	
-	target_type = get_save_file_type_by_suffix (file);
-	if (target_type == NULL) {
-		g_set_error (error, GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
-			     _("Unsupported image type for saving."));
-		return FALSE;
+	/* determine kind of saving */
+	if (g_ascii_strcasecmp (source->format, target->format) == 0 && !source->modified) {
+		success = eog_image_copy_file (source, target, error);
+		direct_copy = success;
 	}
-	
-	source_is_local = (priv->uri != NULL && is_local_uri (priv->uri));
-	source_type = priv->file_type;
-	
-	/* Check for some special cases, so that we use always the least intrusive method 
-	 * for saving the image.
-	 */
-	if ((g_ascii_strcasecmp (target_type, source_type) == 0) &&
-	    !eog_image_is_modified (img))
+	else if ((g_ascii_strcasecmp (source->format, EOG_FILE_FORMAT_JPEG) == 0 && source->exists) ||
+		 (g_ascii_strcasecmp (target->format, EOG_FILE_FORMAT_JPEG) == 0))
 	{
-		/* If image source and target have the same type and
-		 * source is not modified in any way => copy file.
-		 */
-		GnomeVFSResult vfs_result;
-		
-		vfs_result = gnome_vfs_xfer_uri (priv->uri,
-						 uri, 
-						 GNOME_VFS_XFER_DEFAULT /* copy the data */,
-						 GNOME_VFS_XFER_ERROR_MODE_ABORT, /* abort on all errors */
-						 GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE, /* we checked for existing 
-											   file already */
-						 NULL,  /* no progress callback */
-						 NULL);
-		result = (vfs_result == GNOME_VFS_OK);
-		if (!result) {
-			g_set_error (error, EOG_IMAGE_ERROR,
-				     EOG_IMAGE_ERROR_VFS, 
-				     gnome_vfs_result_to_string (vfs_result));
-		}
-		g_print ("copy image file\n");
-	}
-#if HAVE_JPEG
-	else if ((g_ascii_strcasecmp (source_type, "jpeg") == 0) &&
-		 (g_ascii_strcasecmp (target_type, "jpeg") == 0) && 
-		 source_is_local) 
-	{
-		/* If source is a local file and both are jpeg files,
-		 * then use lossless transformation through libjpeg.
-		 */
-		result = eog_image_jpeg_save_lossless (img, file, error);
-
-		g_print ("loseless saving of %s to %s\n", 
-			 gnome_vfs_uri_to_string (priv->uri, GNOME_VFS_URI_HIDE_NONE), file);
-	}
-#if HAVE_EXIF
-	else if ((g_ascii_strcasecmp (target_type, "jpeg") == 0) && priv->exif != NULL) {
-		/* If the target is a jpeg file and the image has EXIF
-		 * information preserve these by saving through
-		 * libjpeg.
-		 */
-		result = eog_image_jpeg_save (img, file, error);
-	}
-#endif
-#endif
-	else {
-		/* In all other cases: Use default save method
-		 * provided by gdk-pixbuf library.
-		 */
-		result = gdk_pixbuf_save (priv->image, file, target_type, error, NULL);
-	}
-	
-	if (result) {
-		/* free the transformation since it's not needed anymore */
-		/* FIXME: You can't undo prev transformations then anymore. */
-		GList *it = priv->undo_stack;
-		for (; it != NULL; it = it->next) 
-			g_object_unref (G_OBJECT (it->data));
-		
-		g_list_free (priv->undo_stack);
-		g_object_unref (priv->trans);
-		priv->trans = NULL;
-		priv->undo_stack = NULL;
-		priv->modified = FALSE;
+		success = eog_image_jpeg_save_file (img, tmpfile, source, target, error);
 	}
 
-	g_free (target_type);
+	if (!success && (*error == NULL)) {
+		success = gdk_pixbuf_save (priv->image, tmpfile, target->format, error, NULL);
+	}
 
-	return result;
+	if (success && !direct_copy) { /* not required if we alredy copied the file directly */
+		/* try to move result file to target uri */
+		success = tmp_file_move_to_uri (tmpfile, target->uri, target->overwrite, error);
+	}
+
+	if (success) {
+		/* update image information to new uri */
+		eog_image_reset_modifications (img);
+		eog_image_link_with_target (img, target);
+	}
+
+	tmp_file_delete (tmpfile);
+	g_free (tmpfile);
+
+	return success;
 }
 
+/* only for compability reasons, use eog_image_save[_as]_by_info. */
+gboolean
+eog_image_save (EogImage *img, GnomeVFSURI *uri, GdkPixbufFormat *format, GError **error)
+{
+	EogImageSaveInfo *source = NULL;
+	EogImageSaveInfo *target = NULL;
+	gboolean success = FALSE;
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+
+	source = eog_image_save_info_from_image (img);
+
+	if (!gnome_vfs_uri_equal (img->priv->uri, uri)) {
+		char *txt_uri = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+		target = eog_image_save_info_from_uri (txt_uri, format);
+		g_free (txt_uri);
+	}
+
+	if (source != NULL && target != NULL) {
+		success = eog_image_save_as_by_info (img, source, target, error);
+	}
+	else if (source != NULL) {
+		success = eog_image_save_by_info (img, source, error);
+	}
+
+	return success;
+}
+
+
 /*
  * This function is extracted from 
  * File: nautilus/libnautilus-private/nautilus-file.c
@@ -1472,7 +1668,7 @@ eog_image_get_caption (EogImage *img)
 	
 	if (!validated && !g_utf8_validate (name, -1, NULL)) {
 		if (name == NULL) {
-			name = "[Invalid Unicode]";
+			name = g_strdup ("[Invalid Unicode]");
 		}
 		else {
 			utf8_name = eel_make_valid_utf8 (name);
@@ -1521,8 +1717,13 @@ eog_image_free_mem_private (EogImage *image)
 			priv->exif = NULL;
 		}
 #endif
+		
+		if (priv->exif_chunk != NULL) {
+			g_free (priv->exif_chunk);
+			priv->exif_chunk = NULL;
+		}
+		priv->exif_chunk_len = 0;
 
-		priv->load_status = EOG_IMAGE_LOAD_STATUS_NONE;
 		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 	}
 }
@@ -1574,6 +1775,25 @@ eog_image_cancel_load (EogImage *img)
 	g_mutex_unlock (priv->status_mutex);
 }
 
+gboolean 
+eog_image_has_metadata (EogImage *img)
+{
+	EogImagePrivate *priv;
+	gboolean has_metadata;
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+
+	priv = img->priv;
+
+	has_metadata = ((priv->exif_chunk != NULL) || (priv->iptc_chunk != NULL));
+
+#if HAVE_EXIF
+	has_metadata |= (priv->exif != NULL);
+#endif	
+
+	return has_metadata;
+}
+
 gpointer
 eog_image_get_exif_information (EogImage *img)
 {
@@ -1634,3 +1854,11 @@ eog_image_get_bytes (EogImage *img)
 	
 	return img->priv->bytes;
 }
+
+void
+eog_image_modified (EogImage *img)
+{
+	g_return_if_fail (EOG_IS_IMAGE (img));
+
+	g_signal_emit (G_OBJECT (img), eog_image_signals [SIGNAL_IMAGE_CHANGED], 0);
+}
