@@ -31,9 +31,6 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
-#include <bonobo-activation/bonobo-activation.h>
-#include <bonobo/Bonobo.h>
-#include <bonobo/bonobo-window.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -57,6 +54,7 @@
 #include "eog-hig-dialog.h"
 #include "eog-uri-converter.h"
 #include "eog-save-as-dialog-helper.h"
+#include "eog-pixbuf-util.h"
 
 /* Default size for windows */
 
@@ -66,12 +64,6 @@
 #define RECENT_FILES_GROUP         "Eye of Gnome"
 #define EOG_WINDOW_DND_POPUP_PATH  "/popups/dragndrop"
 
-#define PROPERTY_WINDOW_STATUS "window/status"
-#define PROPERTY_WINDOW_TITLE  "window/title"
-#define PROPERTY_WINDOW_WIDTH  "window/width"
-#define PROPERTY_WINDOW_HEIGHT "window/height"
-#define PROPERTY_IMAGE_PROGRESS "image/progress"
-
 #define EOG_STOCK_ROTATE_90    "eog-stock-rotate-90"
 #define EOG_STOCK_ROTATE_270   "eog-stock-rotate-270"
 #define EOG_STOCK_ROTATE_180   "eog-stock-rotate-180"
@@ -110,19 +102,15 @@ struct _EogWindowPrivate {
 	EggRecentViewGtk    *recent_view;
 
 	/* gconf notification ids */
-	guint view_statusbar_id;
-	guint view_toolbar_id;
+	guint interp_type_notify_id;
+	guint transparency_notify_id;
+	guint trans_color_notify_id;
 
 	/* signal ids */
 	guint sig_id_list_prepared;
 	guint sig_id_progress;
 	guint sig_id_loading_finished;
 	guint sig_id_loading_failed;
-
-	/* deprecated */
-	BonoboControlFrame  *ctrl_frame;
-	BonoboUIComponent   *ui_comp;
-	GtkWidget           *ctrl_widget;
 };
 
 enum {
@@ -287,8 +275,11 @@ static void
 verb_EditPreferences_cb (GtkAction *action, gpointer data)
 {
 	GConfClient *client;
+	EogWindowPrivate *priv;
 
-	client = EOG_WINDOW (data)->priv->client;
+	priv = EOG_WINDOW (data)->priv;
+
+	client = priv->client;
 
 	eog_preferences_show (client);
 }
@@ -438,20 +429,67 @@ verb_ZoomFit_cb (GtkAction *action, gpointer data)
 	}
 }
 
+static void
+slideshow_hide_cb (GtkWidget *widget, gpointer data)
+{
+	EogImage *last_image;
+	EogFullScreen *fs;
+	EogWindow *window;
+	EogWindowPrivate *priv;
+
+	fs = EOG_FULL_SCREEN (widget);
+	window = EOG_WINDOW (data);
+	priv = window->priv;
+
+	last_image = eog_full_screen_get_last_image (fs);
+	
+	if (last_image != NULL) {
+		eog_wrap_list_set_current_image (EOG_WRAP_LIST (priv->wraplist), last_image, TRUE);
+		g_object_unref (last_image);
+	}
+
+	gtk_widget_destroy (widget);
+}
+
 static void
 verb_FullScreen_cb (GtkAction *action, gpointer data)
 {
 	EogWindowPrivate *priv;
 	GtkWidget *fs;
+	EogImageList *list = NULL;
+	EogImage *start_image = NULL;
+	int n_selected;
 
 	g_return_if_fail (EOG_IS_WINDOW (data));
 
 	priv = EOG_WINDOW (data)->priv;
 
-	fs = eog_full_screen_new (priv->image_list, NULL);
-	g_signal_connect (G_OBJECT (fs), "hide", G_CALLBACK (gtk_widget_destroy), NULL);
+	n_selected = eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist));
+	if (n_selected == 1) {
+		list = g_object_ref (priv->image_list);
+		start_image = eog_wrap_list_get_first_selected_image (EOG_WRAP_LIST (priv->wraplist));
+	}
+	else if (n_selected > 1) {
+		GList *l;
+		l = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (priv->wraplist));
 
-	gtk_widget_show_all (fs);
+		list = eog_image_list_new_from_glist (l);
+
+		g_list_foreach (l, (GFunc) g_object_unref, NULL);
+		g_list_free (l);
+	}
+
+	if (list != NULL) {
+		fs = eog_full_screen_new (list, start_image);
+		g_signal_connect (G_OBJECT (fs), "hide", G_CALLBACK (slideshow_hide_cb), EOG_WINDOW (data));
+
+		gtk_widget_show_all (fs);
+
+		g_object_unref (list);
+	}
+
+	if (start_image != NULL) 
+		g_object_unref (start_image);
 }
 
 static void
@@ -594,7 +632,8 @@ save_error (SaveData *data)
 		header = g_strdup_printf (_("Overwrite file %s?"), uri_str);
 		detail = _("File exists. Do you want to overwrite it?");
 
-		dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR,
+		dlg = eog_hig_dialog_new (GTK_WINDOW (data->window), 
+					  GTK_STOCK_DIALOG_ERROR,
 					  header, detail,
 					  TRUE);
 
@@ -609,7 +648,8 @@ save_error (SaveData *data)
 	else {
 		header = g_strdup_printf (_("Error on saving %s."), eog_image_get_caption (image));
 		
-		dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR,
+		dlg = eog_hig_dialog_new (GTK_WINDOW (data->window), 
+					  GTK_STOCK_DIALOG_ERROR,
 					  header, detail,
 					  TRUE);
 	
@@ -619,8 +659,6 @@ save_error (SaveData *data)
 		gtk_dialog_set_default_response (GTK_DIALOG (dlg), EOG_SAVE_RESPONSE_SKIP);
 	}
 	
-	gtk_window_set_transient_for (GTK_WINDOW (dlg), GTK_WINDOW (data->window));
-	gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_widget_show_all (dlg);
 
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -930,18 +968,90 @@ save_as_image_list (gpointer user_data)
 	return NULL;
 }
 
+/* Asks user for a file location to save an image there. Returns the save target uri
+ * and the image format. The uri parameter is set to NULL if the user canceled the
+ * dialog.
+ */
+static void
+save_as_file_selection_dialog (EogWindow *window, char *folder_uri, char **uri, GdkPixbufFormat **format)
+{
+	GtkWidget *dlg;
+	gboolean success = FALSE;
+	gint response;
+
+	g_return_if_fail (uri != NULL);
+	g_return_if_fail (format != NULL);
+
+	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_SAVE);
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg),
+						 folder_uri);
+	while (!success) {
+		success = TRUE;
+		if (*uri != NULL) {
+			g_free (*uri);
+			*uri = NULL;
+		}
+
+		gtk_widget_show_all (dlg);
+		response = gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_hide (dlg);
+
+		if (response == GTK_RESPONSE_OK) {
+			*format = eog_file_selection_get_format (EOG_FILE_SELECTION (dlg));
+			*uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dlg));
+
+			if (*format == NULL && *uri != NULL) {
+				*format = eog_pixbuf_get_format_by_uri (*uri);
+			}
+				
+			success = (*format != NULL && *uri != NULL);
+		}
+		
+		if (!success) {
+			GtkWidget *err_dlg;
+			char *header;
+			char *detail;
+			char *short_name;
+			char *uesc_uri;
+
+			uesc_uri = gnome_vfs_unescape_string_for_display (*uri);
+			short_name = g_path_get_basename (uesc_uri);
+
+			header = g_strdup_printf (_("Couldn't determine file format of %s"), short_name);
+			detail = _("Please use an appropriate filename suffix or select a file format.");
+			
+			g_free (uesc_uri);
+			g_free (short_name);
+
+			err_dlg = eog_hig_dialog_new (GTK_WINDOW (window),
+						      GTK_STOCK_DIALOG_ERROR,
+						      header, detail,
+						      TRUE);
+			
+			gtk_dialog_add_button (GTK_DIALOG (err_dlg), _("Retry"), GTK_RESPONSE_OK);
+			gtk_dialog_add_button (GTK_DIALOG (err_dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+			gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
+			gtk_widget_show_all (err_dlg);
+			
+			response = gtk_dialog_run (GTK_DIALOG (err_dlg));
+			gtk_widget_destroy (err_dlg);
+			g_free (header);
+		}
+	}
+
+	gtk_widget_destroy (dlg);
+}
+
 static void
 save_as_single_image (EogWindow *window, EogImage *image)
 {
 	EogWindowPrivate *priv;
-	GtkWidget *dlg;
 	char *uri = NULL;
 	SaveData *data;
 	GnomeVFSURI *img_uri;
 	GnomeVFSURI *parent;
 	char *folder_uri = NULL;
 	GdkPixbufFormat *format = NULL;
-	gint response;
 	GtkWidget *button;
 
 	g_return_if_fail (EOG_IS_WINDOW (window));
@@ -956,23 +1066,14 @@ save_as_single_image (EogWindow *window, EogImage *image)
 	gnome_vfs_uri_unref (img_uri);
 	gnome_vfs_uri_unref (parent);
 
-	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_SAVE);
-	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg),
-						 folder_uri);
+	save_as_file_selection_dialog (window, folder_uri, &uri, &format);
 	g_free (folder_uri);
 
-	gtk_widget_show_all (dlg);
-	response = gtk_dialog_run (GTK_DIALOG (dlg));
-	if (response == GTK_RESPONSE_OK) {
-		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dlg));
-		format = eog_file_selection_get_format (EOG_FILE_SELECTION (dlg));
-	}
-
-	gtk_widget_destroy (dlg);
-
 	if (uri == NULL)
 		return;
 
+	g_assert (uri != NULL && format != NULL);
+
 	data = g_new0 (SaveData, 1);
 	g_assert (data != NULL);
 
@@ -1057,11 +1158,11 @@ save_as_multiple_images (EogWindow *window, GList *images)
 		if (!success) {
 			GtkWidget *error_dlg;
 
-			error_dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR,
+			error_dlg = eog_hig_dialog_new (GTK_WINDOW (window),
+							GTK_STOCK_DIALOG_ERROR,
 							_("Error on saving images."),
 							error->message,
 							TRUE);
-			gtk_window_set_transient_for (GTK_WINDOW (error_dlg), GTK_WINDOW (window));
 			gtk_dialog_add_button (GTK_DIALOG (error_dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
 			gtk_dialog_run (GTK_DIALOG (error_dlg));
 			gtk_widget_destroy (error_dlg);
@@ -1293,18 +1394,16 @@ show_delete_confirm_dialog (EogWindow *window, int n_images)
 	char *header;
 	int response;
 
-	header = g_strdup_printf (_("Do you really want to move %i images to trash?"), n_images);
+	header = g_strdup_printf (ngettext ("Do you really want to move %i image to trash?", 
+					    "Do you really want to move %i images to trash?", n_images), n_images);
 
-	dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_WARNING,
+	dlg = eog_hig_dialog_new (GTK_WINDOW (window), GTK_STOCK_DIALOG_WARNING,
 				  header, NULL, TRUE);
 	g_free (header);
 
 	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_DELETE, GTK_RESPONSE_OK);
 	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
-
-	gtk_window_set_transient_for (GTK_WINDOW (dlg), GTK_WINDOW (window));
-	gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_widget_show_all (dlg);
 
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -1409,14 +1508,12 @@ verb_Delete_cb (GtkAction *action, gpointer data)
 			
 			header = g_strdup_printf (_("Error on deleting image %s"), eog_image_get_caption (image));
 			
-			dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_WARNING, header, 
+			dlg = eog_hig_dialog_new (GTK_WINDOW (window), 
+						  GTK_STOCK_DIALOG_WARNING, header, 
 						  error->message, TRUE);
 			
 			gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
 			gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
-			
-			gtk_window_set_transient_for (GTK_WINDOW (dlg), GTK_WINDOW (window));
-			gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER_ON_PARENT);
 			gtk_widget_show_all (dlg);
 			
 			gtk_dialog_run (GTK_DIALOG (dlg));
@@ -1440,6 +1537,100 @@ verb_Delete_cb (GtkAction *action, gpointer data)
 
 /* ========================================================================= */
 
+static void
+interp_type_changed_cb (GConfClient *client,
+			guint        cnxn_id,
+			GConfEntry  *entry,
+			gpointer     user_data)
+{
+	EogWindowPrivate *priv;
+	gboolean interpolate = TRUE;
+
+	g_return_if_fail (EOG_IS_WINDOW (user_data));
+
+	priv = EOG_WINDOW (user_data)->priv;
+
+	if (!EOG_IS_SCROLL_VIEW (priv->scroll_view)) return;
+
+	if (entry->value != NULL && entry->value->type == GCONF_VALUE_BOOL) {
+		interpolate = gconf_value_get_bool (entry->value);
+	}
+
+	eog_scroll_view_set_antialiasing (EOG_SCROLL_VIEW (priv->scroll_view), interpolate);
+}
+
+static void
+transparency_changed_cb (GConfClient *client,
+			 guint        cnxn_id,
+			 GConfEntry  *entry,
+			 gpointer     user_data)
+{
+	EogWindowPrivate *priv;
+	const char *value = NULL;
+
+	g_return_if_fail (EOG_IS_WINDOW (user_data));
+
+	priv = EOG_WINDOW (user_data)->priv;
+	if (!EOG_IS_SCROLL_VIEW (priv->scroll_view)) return;
+
+	if (entry->value != NULL && entry->value->type == GCONF_VALUE_STRING) {
+		value = gconf_value_get_string (entry->value);
+	}
+
+	if (g_strcasecmp (value, "COLOR") == 0) {
+		GdkColor color;
+		char *color_str;
+
+		color_str = gconf_client_get_string (priv->client,
+						     EOG_CONF_VIEW_TRANS_COLOR, NULL);
+		if (gdk_color_parse (color_str, &color)) {
+			eog_scroll_view_set_transparency (EOG_SCROLL_VIEW (priv->scroll_view),
+							  TRANSP_COLOR, &color);
+		}
+	}
+	else if (g_strcasecmp (value, "CHECK_PATTERN") == 0) {
+		eog_scroll_view_set_transparency (EOG_SCROLL_VIEW (priv->scroll_view),
+						  TRANSP_CHECKED, 0);
+	}
+	else {
+		eog_scroll_view_set_transparency (EOG_SCROLL_VIEW (priv->scroll_view),
+						  TRANSP_BACKGROUND, 0);
+	}
+}
+
+static void
+trans_color_changed_cb (GConfClient *client,
+			guint        cnxn_id,
+			GConfEntry  *entry,
+			gpointer     user_data)
+{
+	EogWindowPrivate *priv;
+	GdkColor color;
+	char *value;
+	const char *color_str;
+
+	g_return_if_fail (EOG_IS_WINDOW (user_data));
+
+	priv = EOG_WINDOW (user_data)->priv;
+	if (!EOG_IS_SCROLL_VIEW (priv->scroll_view)) return;
+
+	value = gconf_client_get_string (priv->client, EOG_CONF_VIEW_TRANSPARENCY, NULL);
+
+	if (g_strcasecmp (value, "COLOR") != 0) return;
+
+	if (entry->value != NULL && entry->value->type == GCONF_VALUE_STRING) {
+		color_str = gconf_value_get_string (entry->value);
+
+		if (gdk_color_parse (color_str, &color)) {
+			eog_scroll_view_set_transparency (EOG_SCROLL_VIEW (priv->scroll_view),
+							  TRANSP_COLOR, &color);
+		}
+	}
+}
+
+/* ========================================================================= */
+
+#if 0
 static void
 activate_uri_cb (BonoboControlFrame *control_frame, const char *uri, gboolean relative, gpointer data)
 {
@@ -1455,6 +1646,7 @@ activate_uri_cb (BonoboControlFrame *control_frame, const char *uri, gboolean re
 
 	g_signal_emit (G_OBJECT (window), eog_window_signals[SIGNAL_OPEN_URI_LIST], 0, list);
 }
+#endif
 
 static void
 open_recent_cb (GtkWidget *widget, const EggRecentItem *item, gpointer data)
@@ -1530,11 +1722,6 @@ eog_window_destroy (GtkObject *object)
 
 	window_list = g_list_remove (window_list, window);
 
-	if (priv->ctrl_frame != NULL) {
-		bonobo_object_unref (BONOBO_OBJECT (priv->ctrl_frame));
-		priv->ctrl_frame = NULL;
-	}
-
 	if (priv->recent_view != NULL) {
 		g_object_unref (priv->recent_view);
 		priv->recent_view = NULL;
@@ -1547,11 +1734,12 @@ eog_window_destroy (GtkObject *object)
 
 	/* Clean up GConf-related stuff */
 	if (priv->client) {
-		gconf_client_notify_remove (priv->client, priv->view_toolbar_id);
-		priv->view_toolbar_id = 0;
-
-		gconf_client_notify_remove (priv->client, priv->view_statusbar_id);
-		priv->view_statusbar_id = 0;
+		gconf_client_notify_remove (priv->client, priv->interp_type_notify_id);
+		gconf_client_notify_remove (priv->client, priv->transparency_notify_id);
+		gconf_client_notify_remove (priv->client, priv->trans_color_notify_id);
+		priv->interp_type_notify_id = 0;
+		priv->transparency_notify_id = 0;
+		priv->trans_color_notify_id = 0;
 
 		gconf_client_remove_dir (priv->client, EOG_CONF_DIR, NULL);
 		g_object_unref (G_OBJECT (priv->client));
@@ -1643,8 +1831,23 @@ eog_window_init (EogWindow *window)
 	gconf_client_add_dir (priv->client, EOG_CONF_DIR,
 			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
 
+	priv->interp_type_notify_id =
+		gconf_client_notify_add (priv->client,
+					 EOG_CONF_VIEW_INTERPOLATE,
+					 interp_type_changed_cb,
+					 window, NULL, NULL);
+	priv->transparency_notify_id =
+		gconf_client_notify_add (priv->client,
+					 EOG_CONF_VIEW_TRANSPARENCY,
+					 transparency_changed_cb,
+					 window, NULL, NULL);
+	priv->trans_color_notify_id =
+		gconf_client_notify_add (priv->client,
+					 EOG_CONF_VIEW_TRANS_COLOR,
+					 trans_color_changed_cb,
+					 window, NULL, NULL);
+
 	window_list = g_list_prepend (window_list, window);
-	priv->ctrl_widget = NULL;
 
 	priv->desired_width = -1;
 	priv->desired_height = -1;
@@ -1697,7 +1900,7 @@ eog_window_has_contents (EogWindow *window)
 {
 	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
 
-	return (window->priv->ctrl_frame != NULL);
+	return (eog_image_list_length (window->priv->image_list) > 0);
 }
 
 /* Drag_data_received handler for windows */
@@ -1750,11 +1953,13 @@ set_drag_dest (EogWindow *window)
 			   GDK_ACTION_COPY | GDK_ACTION_ASK);
 }
 
+#if 0
 static void
 widget_realized_cb (GtkWidget *widget, gpointer data)
 {
 	adapt_window_size (EOG_WINDOW (data), 250, 250);
 }
+#endif
 
 static void
 update_ui_visibility (EogWindow *window)
@@ -1835,6 +2040,7 @@ handle_image_selection_changed (EogWrapList *list, EogWindow *window)
 	EogImage *image;
 	gchar *str;
 	gint nimg, nsel;
+	char *title = NULL; 
 
 	priv = window->priv;
 
@@ -1870,8 +2076,24 @@ handle_image_selection_changed (EogWrapList *list, EogWindow *window)
 	nimg = eog_image_list_length (priv->image_list);
 	nsel = eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist));
 	
+	/* Images: (n_selected_images) / (n_total_images) */
 	str = g_strdup_printf (_("Images: %i/%i"), nsel, nimg);
 	gnome_appbar_set_status (GNOME_APPBAR (priv->statusbar), str);
+
+	/* update window title */
+	if (image != NULL) {
+		GnomeVFSURI *uri;
+		
+		uri = eog_image_get_uri (image);
+		title = gnome_vfs_uri_extract_short_name (uri);
+		gnome_vfs_uri_unref (uri);
+	}
+	else {
+		title = g_strdup (_("Eye of Gnome"));
+	}
+	gtk_window_set_title (GTK_WINDOW (window), title);
+		
+	g_free (title);
 	g_free (str);
 }
 
@@ -1899,7 +2121,7 @@ add_eog_icon_factory (void)
 	GnomeProgram *program;
 
 	factory = gtk_icon_factory_new ();
-	program = gnome_program_get ();
+	program = gnome_program_get (); /* don't free this it's static */
 	g_assert (program != NULL);
 
 	while (eog_stock_items[i].stock_id != NULL) {
@@ -1929,16 +2151,15 @@ add_eog_icon_factory (void)
 
 	gtk_icon_factory_add_default (factory);
 	g_object_unref (factory);
-	g_object_unref (program);
 }
 
 /* UI<->function mapping */
 /* Normal items */
 static GtkActionEntry action_entries_window[] = {
-  { "FileMenu", NULL, "_File" },
-  { "EditMenu", NULL, "_Edit" },
-  { "ViewMenu", NULL, "_View" },
-  { "HelpMenu", NULL, "_Help" },
+  { "FileMenu", NULL, N_("_File") },
+  { "EditMenu", NULL, N_("_Edit") },
+  { "ViewMenu", NULL, N_("_View") },
+  { "HelpMenu", NULL, N_("_Help") },
   { "FileNewWindow",   GTK_STOCK_NEW,   N_("_New"),      "<control>N",  N_("Open a new window"),            G_CALLBACK (verb_FileNewWindow_cb) },
   { "FileOpen",        GTK_STOCK_OPEN,  N_("_Open..."),  "<control>O",  N_("Open a file"),                  G_CALLBACK (verb_FileOpen_cb) },
   { "FileDirOpen",     GTK_STOCK_OPEN,  N_("Open _Directory..."), "<control><shift>O", N_("Open a directory"), G_CALLBACK (verb_DirOpen_cb) },
@@ -1946,7 +2167,7 @@ static GtkActionEntry action_entries_window[] = {
   { "FileExit",        GTK_STOCK_QUIT,  N_("_Quit"),     "<control>Q",  N_("Close all windows and quit"),   G_CALLBACK (verb_FileExit_cb) },
   { "EditPreferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, N_("Preferences for Eye of Gnome"), G_CALLBACK (verb_EditPreferences_cb) },
   { "HelpManual",      GTK_STOCK_HELP,  N_("_Contents"), "F1",          N_("Help On this application"),     G_CALLBACK (verb_HelpContent_cb) },
-  { "HelpAbout",       NULL,            N_("_About"),    NULL,          N_("About this application"),       G_CALLBACK (verb_HelpAbout_cb) }
+  { "HelpAbout",       GNOME_STOCK_ABOUT, N_("_About"),	NULL, N_("About this application"),       G_CALLBACK (verb_HelpAbout_cb) }
 };
 
 /* Toggle items */
@@ -2011,11 +2232,13 @@ eog_window_construct_ui (EogWindow *window)
 	
 	/* build menu and toolbar */
 	priv->actions_window = gtk_action_group_new ("MenuActionsWindow");
+	gtk_action_group_set_translation_domain (priv->actions_window, PACKAGE);
 	gtk_action_group_add_actions (priv->actions_window, action_entries_window, G_N_ELEMENTS (action_entries_window), window);
 	gtk_action_group_add_toggle_actions (priv->actions_window, toggle_entries_window, G_N_ELEMENTS (toggle_entries_window), window);
 	gtk_ui_manager_insert_action_group (priv->ui_mgr, priv->actions_window, 0);
 
 	priv->actions_image = gtk_action_group_new ("MenuActionsImage");
+	gtk_action_group_set_translation_domain (priv->actions_image, PACKAGE);
 	gtk_action_group_add_actions (priv->actions_image, action_entries_image, G_N_ELEMENTS (action_entries_image), window);
 	gtk_ui_manager_insert_action_group (priv->ui_mgr, priv->actions_image, 0);
 
@@ -2099,22 +2322,6 @@ eog_window_construct_ui (EogWindow *window)
 
 	gtk_box_pack_start (GTK_BOX (priv->box), priv->vpane, TRUE, TRUE, 0);
 
-#if 0
-	/* We connect to the realize signal here, because we must know
-	 * when all the child widgets of an eog window have determined
-	 * their size in order to get a working adapt_window_size
-	 * function. It should be sufficient to connect to the
-	 * statusbar here, because it is one of the deepest widgets in
-	 * the widget hierarchy tree.  FIXME: if we allow hideable
-	 * status-/toolbars then this must be reworked.
-	 */
-	g_signal_connect_after (G_OBJECT (priv->statusbar),
-				"realize", G_CALLBACK (widget_realized_cb), window);
-	gtk_widget_show (GTK_WIDGET (priv->statusbar));
-
-	/* add control frame interface */
-	priv->ctrl_frame = NULL;
-#endif
 	set_drag_dest (window);
 
 	/* set default geometry */
@@ -2142,6 +2349,38 @@ eog_window_construct_ui (EogWindow *window)
 	update_ui_visibility (window);
 }
 
+static void
+eog_window_update_properties (EogWindow *window)
+{
+	GConfEntry *entry;
+	EogWindowPrivate *priv;
+
+	g_return_if_fail (EOG_IS_WINDOW (window));
+
+	priv = window->priv;
+
+	entry = gconf_client_get_entry (priv->client, EOG_CONF_VIEW_INTERPOLATE, NULL, TRUE, NULL);
+	if (entry != NULL) {
+		interp_type_changed_cb (priv->client, 0, entry, window);
+		gconf_entry_free (entry);
+		entry = NULL;
+	}
+
+	entry = gconf_client_get_entry (priv->client, EOG_CONF_VIEW_TRANSPARENCY, NULL, TRUE, NULL);
+	if (entry != NULL) {
+		transparency_changed_cb (priv->client, 0, entry, window);
+		gconf_entry_free (entry);
+		entry = NULL;
+	}
+
+	entry = gconf_client_get_entry (priv->client, EOG_CONF_VIEW_TRANS_COLOR, NULL, TRUE, NULL);
+	if (entry != NULL) {
+		trans_color_changed_cb (priv->client, 0, entry, window);
+		gconf_entry_free (entry);
+		entry = NULL;
+	}
+}
+
 /**
  * eog_window_new:
  * @void:
@@ -2158,6 +2397,7 @@ eog_window_new (void)
 	window = EOG_WINDOW (g_object_new (EOG_TYPE_WINDOW, "title", _("Eye of Gnome"), NULL));
 
 	eog_window_construct_ui (window);
+	eog_window_update_properties (window);
 
 	return GTK_WIDGET (window);
 }
@@ -2178,7 +2418,7 @@ eog_window_close (EogWindow *window)
 	gtk_widget_destroy (GTK_WIDGET (window));
 
 	if (!window_list)
-		bonobo_main_quit ();
+		gtk_main_quit ();
 }
 
 static void
