@@ -23,7 +23,7 @@
 #ifndef EOG_WINDOW_H
 #define EOG_WINDOW_H
 
-#include <bonobo.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS 
 
@@ -41,7 +41,8 @@ typedef enum {
 	EOG_WINDOW_ERROR_CONTROL_NOT_FOUND,
 	EOG_WINDOW_ERROR_NO_PERSIST_FILE_INTERFACE,
 	EOG_WINDOW_ERROR_IO,
-	EOG_WINDOW_ERROR_UNKNOWN,
+	EOG_WINDOW_ERROR_TRASH_NOT_FOUND,
+	EOG_WINDOW_ERROR_UNKNOWN
 } EogWindowError;
 #define EOG_WINDOW_ERROR eog_window_error_quark ()
 
@@ -52,14 +53,14 @@ typedef struct _EogWindowPrivate  EogWindowPrivate;
 
 
 struct _EogWindow {
-	BonoboWindow win;
+	GtkWindow win;
 
 	/* Private data */
 	EogWindowPrivate *priv;
 };
 
 struct _EogWindowClass {
-	BonoboWindowClass parent_class;
+	GtkWindowClass parent_class;
 
 	void (* open_uri_list) (EogWindow *window, GSList *text_uri_list);
 	void (* new_window) (EogWindow *window);
